@@ -353,11 +353,18 @@ namespace Lum {
       Redraw();
     }
 
-    void Edit::SetColorSheme(ColorShemeRef colorSheme)
+    void Edit::SetColorSheme(const ColorShemeRef& colorSheme)
     {
       this->colorSheme=colorSheme;
     }
 
+    void Edit::SetFont(OS::Font* font)
+    {
+      assert(!inited); // TODO: ake it always setable!
+
+      this->font=font;
+    }
+
     void Edit::CalcSize()
     {
       spaceWidth=font->StringWidth(L" ",OS::Font::normal);
