@@ -19,6 +19,7 @@
 
 #include <Lum/TableView.h>
 
+#include <cmath>
 #include <cstdlib>
 #include <memory>
 
@@ -82,7 +83,10 @@ namespace Lum {
     currentPos(1),
     scrolling(false),
     scrollRefresh(false),
-    timer(new Model::Action()),
+    scrollTimerAction(new Model::Action()),
+    cx(0),
+    cy(0),
+    mouseTimerAction(new Model::Action()),
     tableWidth(0),
     rowHeight(0),
     formatProvider(new FormatProvider()),
@@ -93,14 +97,16 @@ namespace Lum {
     SetPrefs(::Lum::prefs);
     SetController(::Lum::controller);
 
-    AttachModel(timer);
     AttachModel(updateScrollerAction);
+    AttachModel(scrollTimerAction);
+    AttachModel(mouseTimerAction);
   }
 
   TableView::~TableView()
   {
+    UnattachModel(mouseTimerAction);
+    UnattachModel(scrollTimerAction);
     UnattachModel(updateScrollerAction);
-    UnattachModel(timer);
 
     if (model.Valid()) {
       UnattachModel(model);
@@ -394,7 +400,7 @@ namespace Lum {
     // TableView does know, that GetParent()->GetParent() is normally of type
     // Table and requests width of vertical scroller to resize labels that way
     // that content does not scroll out of view...
-    if (GetParent()!=NULL &&
+    if (firstDraw && GetParent()!=NULL &&
         dynamic_cast<Table*>(GetParent()->GetParent())!=NULL/* &&
         vAdjustment->GetVisible()<hAdjustment->GetTotal()*/) {
       targetWidth-=dynamic_cast<Table*>(GetParent()->GetParent())->GetVerticalScrollerWidth();
@@ -776,7 +782,7 @@ namespace Lum {
   {
     CalculateRowHeight();
 
-    if (model.Valid() && header.Valid() && firstDraw && autoFitColumns) {
+    if (model.Valid() && header.Valid() && autoFitColumns) {
       FitColumns();
     }
 
@@ -852,7 +858,7 @@ namespace Lum {
         scrolling=(currentPos!=vAdjustment->GetTop());
 
         if (scrolling) {
-          Lum::OS::display->AddTimer(0,1000000/25,timer);
+          Lum::OS::display->AddTimer(0,1000000/25,scrollTimerAction);
         }
 
       }
@@ -1125,13 +1131,19 @@ namespace Lum {
     }
 
     if (event.type==OS::MouseEvent::down && PointIsIn(event)) {
-      if (event.button==OS::MouseEvent::button1 || event.button==OS::MouseEvent::button3) {
+      if (event.button==OS::MouseEvent::button1) {
         if (GetCell(event.x,event.y,xc,yc) && selection.Valid() && yc<=model->GetRows()) {
+          cx=event.x;
+          cy=event.y;
+
           selection->SelectCell(xc,yc);
           OnMouseSelection();
           if (GetWindow()->IsDoubleClicked()) {
             ActionDoubleClick();
           }
+
+          Lum::OS::display->AddTimer(0,1000000/20,mouseTimerAction);
+          mouseTimer.SetToNow();
         }
       }
       else if (event.button==OS::MouseEvent::button4) {
@@ -1143,6 +1155,45 @@ namespace Lum {
 
       return true;
     }
+    else if (event.type==OS::MouseEvent::move && (event.qualifier & OS::MouseEvent::button1)) {
+      Lum::Base::SystemTime now;
+
+      now.Sub(mouseTimer);
+
+      int step=((cy-event.y)*1000000)/(now.GetMicroseconds()+1000000*now.GetTime());
+
+      if (abs(step)<5) {
+        return true;
+      }
+
+      if (step>0) {
+        step=(int)sqrt(step);
+      }
+      else {
+        step=-(int)sqrt(abs(step));
+      }
+
+      if (step>0) {
+        vAdjustment->SetTop(vAdjustment->GetTop()+step);
+      }
+      else if (step<0) {
+        if (abs(step)<vAdjustment->GetTop()-1) {
+          vAdjustment->SetTop(vAdjustment->GetTop()+step);
+        }
+        else {
+          vAdjustment->SetTop(1);
+        }
+      }
+
+      cx=event.x;
+      cy=event.y;
+      mouseTimer.SetToNow();
+
+      return true;
+    }
+    else if (event.type==OS::MouseEvent::up && PointIsIn(event)) {
+      Lum::OS::display->RemoveTimer(mouseTimerAction);
+    }
 
     return false;
   }
@@ -1210,11 +1261,18 @@ namespace Lum {
       else if (model==selection) {
         Redraw();
       }
-      if (model==timer && timer->IsFinished()) {
+      if (model==scrollTimerAction && scrollTimerAction->IsFinished()) {
         scrollRefresh=true;
         Redraw();
         scrollRefresh=false;
       }
+      else if (model==mouseTimerAction && mouseTimerAction->IsFinished()) {
+        vAdjustment->SetTop(currentPos);
+        scrollRefresh=true;
+        Redraw();
+        scrollRefresh=false;
+        Lum::OS::display->RemoveTimer(scrollTimerAction);
+      }
       if (model==updateScrollerAction && updateScrollerAction->IsFinished()) {
         UpdateScroller();
       }
